from flask import render_template, url_for, request, redirect, flash, Response, send_file, abort
from blog import app, db
from blog.models import User, Post, Portfolio
from blog.forms import RegistrationForm, LoginForm, UpdateAccountForm, PostForm
from flask_login import login_user, logout_user, current_user

@app.route("/")
@app.route("/home")
def home():
  return render_template('home.html', title='Home')

@app.route("/portfolio")
def portfolio():
  items = Portfolio.query.all()
  return render_template('portfolio.html', items=items)

@app.route("/experience")
def experience():
  return render_template('experience.html', title='Experience')

@app.route("/cv")
def cv():
  return render_template('cv.html', title='cv')

@app.route('/download')
def download():
  return send_file('./static/pdf/cv.pdf', as_attachment=True)

@app.route("/blog")
def blog():
  posts=Post.query.all()
  return render_template('blog.html',posts=posts)

@app.route("/post", methods=['GET', 'POST'])
def post():
  form = PostForm()
  if form.validate_on_submit():
    post = Post(title=form.title.data, content=form.content.data, author_id=current_user.id)
    db.session.add(post)
    db.session.commit()
    flash('Success')
    return redirect(url_for('blog'))
  return render_template('post.html', title='New Post', form=form, legend='New Post')

@app.route("/post/update/<int:post_id>", methods=['GET', 'POST'])
def update_post(post_id):
    post = Post.query.get_or_404(post_id)
    if not post.author_id == current_user.id:
        abort(403)
    form = PostForm()
    if form.validate_on_submit():
        post.title = form.title.data
        post.content = form.content.data
        db.session.commit()
        flash("Your post has been updated successfully!")
        return redirect(url_for('post', post_id=post.id))
    elif not request.method != 'GET':
        form.title.data = post.title
        form.content.data = post.content
    return render_template('post.html', title='Update Post', form=form, legend='Update Post')
 
@app.route("/post/delete/<int:post_id>", methods=['GET', 'POST'])
def delete_post(post_id):
    post = Post.query.get_or_404(post_id)
    if not post.author_id == current_user.id:
        abort(403)
    db.session.delete(post)
    db.session.commit()
    flash("Your post has been deleted successfully",'success')
    return redirect(url_for('home'))

@app.route("/account", methods=['GET', 'POST'])
def account():
  form = UpdateAccountForm()
  if form.validate_on_submit():
    current_user.username = form.username.data
    current_user.email = form.email.data
    db.session.commit()
    flash('Your account has been updated!')
    return redirect(url_for('account'))
  elif request.method == 'GET':
    form.username.data = current_user.username
    form.email.data = current_user.email
  return render_template('account.html', title='Account', form=form)

@app.route("/register",methods=['GET','POST'])
def register():
  form = RegistrationForm()
  if form.validate_on_submit():
    user = User(username=form.username.data, email=form.email.data, password=form.password.data)
    db.session.add(user)
    db.session.commit()
    flash('Registration successful!')
    return redirect(url_for('login'))
  return render_template('register.html',title='Register',form=form)

@app.route("/registered")
def registered():
  return render_template('registered.html', title='Thanks!')

@app.route("/login",methods=['GET','POST'])
def login():
  form = LoginForm()
  if form.validate_on_submit():
    user = User.query.filter_by(username=form.username.data).first()
    if user and (user.password, form.password.data):
      login_user(user)
      flash('You\'ve successfully logged in,'+' '+ current_user.username +'!')
      return redirect(url_for('home'))
    else:
      flash('Invalid username or password.')
  return render_template('login.html',title='Login', form=form)

@app.route("/logout")
def logout():
  logout_user()
  flash('You\'re now logged out. Thanks for your visit!')
  return redirect(url_for('home'))
